# -*- perl -*-

# /t/500_full_swarm/501_9-to-1.t - Miniature swarm of 9 seeds and 1 new peer
# $Id: 505_9-to-1.t 10 2008-04-05 22:06:00Z sanko@cpan.org $

system qq[$^X ./t/data/etc/http_miniswarm.pl 9 1];
